# Copyright (C) 2018 Adan Salazar <asalazargaribay@gmail.com>
#
#
# This file is part of GeoSentinel
#
#
# GeoSentinel is free software  you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# GeoSentinel is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with StereoVision.  If not, see <http://www.gnu.org/licenses/>.

"""
Utilities for user interaction with the "geosentinel" package.

Variables:

    * "SENTINEL_ARGUMENTS" - "argparse.ArgumentParser" for working with Sentinel products
    
Functions:

    * "find_files_Sentinel" - Discover Sentinel files in directory
    * "unzip_files_Sentinel" - Unzip Sentinel files in directory
    * "download_sentinel_mages" - Calibrate chessboard images discovered in a folder
    * "check_existing_directory" - Check out if the folder exist. Create folder if it does not exist
    * "check_existing_productType" - Check args if exist
    * "select_products" -  # Select products that their intersection area is less than a certain percentage threshold
    * "combinations_bands_sentinel" - Get combination bands such product
    * "search_band_combination" - Get path and bands for combination such product
    * "find_files_byext_Sentinel" - Discover Sentinel files by extension
"""

#from argparse import ArgumentParser

import os,sys
import zipfile

from collections import OrderedDict

#from osgeo import ogr

#from osgeo import gdal

#from osgeo.gdalconst import *

from sentinelsat.sentinel import SentinelAPI #read_geojson, geojson_to_wkt

from geosentinel.products import SentinelData


def check_existing_productType(product,args,flag):
    if(flag==1):
        product_type=['Sentinel-1', 'sentinel-1', 'Sentinel-2', 'sentinel-2']
        findProduct = product in product_type
        if not findProduct:
            print product + " not exist"
            sys.exit()
    if(flag==2):
        product_type=["all", "SM_SLC", "SM_GRDF", "SM_GRDH", "SM_GRDM", "IW_SLC", "IW_GRDH", "IW_GDRM", "EW_SLC", "EW_GRDH", "EW_GRDM", "WV_SLC", "WV_GRDM", "MSIL1C", "OPER_PRD","RGB"]
        findProduct = product in product_type
        if not findProduct:
            print product + " not exist"
            sys.exit()
    if(flag==3):
        ext="_RGB.tif"
        product = product[:-5]+ext
        product_type = os.listdir(args.folder)
        findProduct = product in product_type
    return findProduct

def combinations_bands_sentinel(sentinel_image,args,products):
    TotImage=len(products)
    #Get Raster
    dataset1 = gdal.Open(products[2], GA_ReadOnly)
    dataset2 = gdal.Open(products[1], GA_ReadOnly)
    dataset3 = gdal.Open(products[0], GA_ReadOnly)
    #Get metadata raster
    projectionfrom = dataset1.GetProjection()
    geotransform = dataset1.GetGeoTransform()
    band1 = dataset1.GetRasterBand(1)
    xsize = band1.XSize
    ysize = band1.YSize
    datatype1 = band1.DataType
    band2 = dataset2.GetRasterBand(1)
    datatype2 = band2.DataType
    band3 = dataset3.GetRasterBand(1)
    datatype3 = band3.DataType
    #Reading the raster values
    gtiff = gdal.GetDriverByName('GTiff')
    output_file=sentinel_image[:-5]+"_"+args.product_output_type+".tif"
    values1 = band1.ReadRaster( 0, 0, xsize, ysize, xsize, ysize, datatype1 )
    values2 = band2.ReadRaster( 0, 0, xsize, ysize, xsize, ysize, datatype2 )
    values3 = band3.ReadRaster( 0, 0, xsize, ysize, xsize, ysize, datatype3 )
    #Combination bands
    output_dataset = gtiff.Create(output_file, xsize, ysize, TotImage,GDT_Int16)
    output_dataset.SetProjection(projectionfrom)
    output_dataset.SetGeoTransform(geotransform)
    output_dataset.GetRasterBand(1).WriteRaster( 0, 0, xsize, ysize, values1 )
    output_dataset.GetRasterBand(2).WriteRaster( 0, 0, xsize, ysize, values2 )
    output_dataset.GetRasterBand(3).WriteRaster( 0, 0, xsize, ysize, values3 )
    output_dataset = None

def search_band_combination(list_files_byext,args):
    for image in list_files_byext:
        sentinel_image=args.folder+"\\"+image
        findProduct=check_existing_productType(image,args, 3)
        if findProduct==True:
            print  args.product_output_type+ " was applied to image... "+image
        else:
            Granule = "\GRANULE"
            path=sentinel_image+Granule
            file = os.listdir(path)
            path=path+ "\\"+file[0]
            Img_Data = "\IMG_DATA"
            path=path+Img_Data
            file = os.listdir(path)
            products=[]
            if args.product_output_type=="RGB":
                ext1="B04.jp2"
                ext2="B03.jp2"
                ext3="B02.jp2"
                raster = [f for f in file if f.endswith(ext1) or f.endswith(ext2) or f.endswith(ext3)]
            for raster in raster:
                product=path+"\\"+raster
                products.append(product)
            print"Applying "+args.product_output_type+" to the image... ",image
            combinations_bands_sentinel(sentinel_image,args,products)

def find_files_byext_Sentinel(files,args):
    findProduct=check_existing_productType(args.product_output_type,args, 2)
    if args.product_output_type == "RGB":
        if args.plattform == 'Sentinel-1' or args.plattform == 'sentinel-1':
            sys.exit()
        if args.plattform == 'Sentinel-2' or args.plattform == 'sentinel-2':
            ext=".SAFE"
    files = [f for f in files if f.endswith(ext)]
    if (len(files) == 0):
        print"Files not found..."
    return files

def find_files_Sentinel(args):
    check_existing_directory(args.folder, 2)
    findProduct=check_existing_productType(args.plattform,args, 1)
    findProduct=check_existing_productType(args.product_type,args, 2)
    files = os.listdir(args.folder)
    if (args.product_type == "all"):
        if args.plattform == 'Sentinel-1' or args.plattform == 'sentinel-1':
            namfil1 = 'S1A_'
            namfil2 = 'S1B_'
        if args.plattform == 'Sentinel-2' or args.plattform == 'sentinel-2':
            namfil1 = 'S2A_'
            namfil2 = 'S2B_'
        files = [f for f in files if f.startswith(namfil1) or f.startswith(namfil2)]
        if (len(files) == 0):
            print"Files not found..."
    else:
        if args.plattform=="Sentinel-1" or args.plattform=="sentinel-1":
            namfil1 = 'S1A_'+ args.product_type
            namfil2 = 'S1B_'+ args.product_type
        if args.plattform=="Sentinel-2" or args.plattform=="sentinel-2":
            namfil1 = 'S2A_'+ args.product_type
            namfil2 = 'S2B_'+ args.product_type
        files = [f for f in files if f.startswith(namfil1) or f.startswith(namfil2)]
        if (len(files) == 0):
            print"Files not found..."
    return files

def unzip_files_Sentinel(list_files,args):
    for image in list_files:
        print"Unzziped... ",image
        File = args.folder + "\\" +image
        zip_ref = zipfile.ZipFile(File) # create zipfile object
        zip_ref.extractall(args.folder) # extract file to dir
        zip_ref.close() # close file

def check_existing_directory(folder,flag):

    if(flag==1):
        if not os.path.exists(folder):
            os.makedirs(folder)
            print folder + " was created"
        else:
            print folder + " exists "
    if(flag == 2):
        if not os.path.exists(folder):
            print folder + " not exist"
            sys.exit()


def select_products( products_list , products_df, user_footprint, threshold ) :
    
   
    products_down = OrderedDict()
    products_down = products_list.copy()
         
    polyfootprint = ogr.CreateGeometryFromWkt(user_footprint)
    areafootprint = polyfootprint.GetArea()
     
    for i in range(len(products_df)):
        
       inputpoly = ogr.CreateGeometryFromWkt(products_df.footprint[i])
        
       intersectionpoly = polyfootprint.Intersection(inputpoly)
        
       intersectionpoly_area = intersectionpoly.GetArea()
    
       por_intersection  = (intersectionpoly_area*100)/areafootprint
                
       if(por_intersection <=  threshold):
           # Deleting element
           del products_down[products_df.uuid[i]]

    return products_down


def download_sentinel_mages(args):
    
            
    check_existing_directory(args.output_folder,1)
    
    products = SentinelData()
    
    api = SentinelAPI(products.username, products.password, products.URL_Sentinel)
    
    products.interest_footprint =  args.footprint
    
    products.init_date = args.init_date
    
    products.end_date = args.end_date
    
    products.product_type = args.product_type
    
    products.platform_name = args.plattform
    
    products.intersection_th = args.overlap
    
        
    products_list = api.query(products.interest_footprint,
                     date = (products.init_date, products.end_date),
                     producttype = products.product_type,
                     platformname = products.platform_name)
    
    products_df = api.to_dataframe(products_list)
        
    products_to_downloading = select_products(products_list, products_df, products.interest_footprint, products.intersection_th ) 
        
   
    os.chdir(args.output_folder)
    api.download_all(products_to_downloading)


    

    
  

